@@ -2,6 +2,6 @@
 W&B callback for CatBoost
 """
 
-from .catboost import log_summary, WandbCallback
+from .catboost import WandbCallback, log_summary
 
 __all__ = ["log_summary", "WandbCallback"]
