@@ -1,22 +1,20 @@
 import os
 
-import six
 import wandb
 from wandb import env
-from wandb.lib import apikey
 
 
 def sagemaker_auth(overrides=None, path=".", api_key=None):
     """Write a secrets.env file with the W&B ApiKey and any additional secrets passed.
 
-    Args:
+    Arguments:
         overrides (dict, optional): Additional environment variables to write
                                     to secrets.env
         path (str, optional): The path to write the secrets file.
     """
 
     settings = wandb.setup().settings
-    current_api_key = apikey.api_key(settings=settings)
+    current_api_key = wandb.wandb_lib.apikey.api_key(settings=settings)
 
     overrides = overrides or dict()
     api_key = overrides.get(env.API_KEY, api_key or current_api_key)
@@ -27,5 +25,5 @@ def sagemaker_auth(overrides=None, path=".", api_key=None):
         )
     overrides[env.API_KEY] = api_key
     with open(os.path.join(path, "secrets.env"), "w") as file:
-        for k, v in six.iteritems(overrides):
-            file.write("{}={}\n".format(k, v))
+        for k, v in overrides.items():
+            file.write(f"{k}={v}\n")
