@@ -3,7 +3,12 @@ import math
 import sys
 import typing as t
 
-from wandb.util import _is_artifact_string, get_module, is_numpy_array
+from wandb.util import (
+    _is_artifact_string,
+    _is_artifact_version_weave_dict,
+    get_module,
+    is_numpy_array,
+)
 
 np = get_module("numpy")  # intentionally not required
 
@@ -59,8 +64,8 @@ class TypeRegistry:
             return NoneType()
 
         # TODO: generalize this to handle other config input types
-        if _is_artifact_string(py_obj):
-            return TypeRegistry.types_by_name().get("artifactVersion")()  # what?
+        if _is_artifact_string(py_obj) or _is_artifact_version_weave_dict(py_obj):
+            return TypeRegistry.types_by_name().get("artifactVersion")()
 
         class_handler = TypeRegistry.types_by_class().get(py_obj.__class__)
         _type = None
@@ -295,6 +300,7 @@ class Type:
         return self is other or (
             isinstance(self, Type)
             and isinstance(other, Type)
+            and self.name == other.name
             and self.params.keys() == other.params.keys()
             and all([self.params[k] == other.params[k] for k in self.params])
         )
