@@ -8,6 +8,7 @@ from typing import TYPE_CHECKING, Optional, Sequence, Type, Union, cast
 import wandb
 from wandb import util
 from wandb._globals import _datatypes_callback
+from wandb.sdk.lib import filesystem
 
 from .wb_value import WBValue
 
@@ -128,7 +129,7 @@ class Media(WBValue):
         file_path = _wb_filename(key, step, id_, extension)
         media_path = os.path.join(self.get_media_subdir(), file_path)
         new_path = os.path.join(self._run.dir, media_path)
-        util.mkdir_exists_ok(os.path.dirname(new_path))
+        filesystem.mkdir_exists_ok(os.path.dirname(new_path))
 
         if self._is_tmp:
             shutil.move(self._path, new_path)
