@@ -1,10 +1,9 @@
-import base64
-import binascii
-import codecs
 import contextlib
 import hashlib
 import os
 import random
+import tempfile
+from dataclasses import dataclass, field
 from typing import (
     IO,
     TYPE_CHECKING,
@@ -20,6 +19,8 @@ from typing import (
 import wandb
 from wandb import env, util
 from wandb.data_types import WBValue
+from wandb.sdk.lib import filesystem
+from wandb.sdk.lib.hashutil import B64MD5, ETag, b64_to_hex_id
 
 if TYPE_CHECKING:
     # need this import for type annotations, but want to avoid circular dependency
@@ -40,55 +41,8 @@ if TYPE_CHECKING:
             pass
 
 
-def md5_string(string: str) -> util.B64MD5:
-    hash_md5 = hashlib.md5()
-    hash_md5.update(string.encode())
-    return base64.b64encode(hash_md5.digest()).decode("ascii")
-
-
-def b64_string_to_hex(string):
-    return binascii.hexlify(base64.standard_b64decode(string)).decode("ascii")
-
-
-def md5_hash_file(path) -> util.RawMD5:
-    hash_md5 = hashlib.md5()
-    with open(path, "rb") as f:
-        for chunk in iter(lambda: f.read(64 * 1024), b""):
-            hash_md5.update(chunk)
-    return hash_md5
-
-
-def md5_hash_files(paths: List[str]) -> util.RawMD5:
-    hash_md5 = hashlib.md5()
-    # Create a mutable copy to sort
-    paths = [path for path in paths]
-    paths.sort()
-    for path in paths:
-        with open(path, "rb") as f:
-            for chunk in iter(lambda: f.read(64 * 1024), b""):
-                hash_md5.update(chunk)
-    return hash_md5
-
-
-def md5_file_b64(path: str) -> util.B64MD5:
-    return base64.b64encode(md5_hash_file(path).digest()).decode("ascii")
-
-
-def md5_files_b64(paths: List[str]) -> util.B64MD5:
-    return base64.b64encode(md5_hash_files(paths).digest()).decode("ascii")
-
-
-def md5_file_hex(path: str) -> util.HexMD5:
-    return md5_hash_file(path).hexdigest()
-
-
-def bytes_to_hex(bytestr):
-    # Works in python2 / python3
-    return codecs.getencoder("hex")(bytestr)[0]
-
-
 class ArtifactManifest:
-    entries: Dict[str, "ArtifactEntry"]
+    entries: Dict[str, "ArtifactManifestEntry"]
 
     @classmethod
     # TODO: we don't need artifact here.
@@ -116,10 +70,10 @@ class ArtifactManifest:
         self.entries = entries or {}
 
     def to_manifest_json(self):
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def digest(self):
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def add_entry(self, entry):
         if (
@@ -129,7 +83,7 @@ class ArtifactManifest:
             raise ValueError("Cannot add the same path twice: %s" % entry.path)
         self.entries[entry.path] = entry
 
-    def get_entry_by_path(self, path: str) -> Optional["ArtifactEntry"]:
+    def get_entry_by_path(self, path: str) -> Optional["ArtifactManifestEntry"]:
         return self.entries.get(path)
 
     def get_entries_in_directory(self, directory):
@@ -142,14 +96,22 @@ class ArtifactManifest:
         ]
 
 
-class ArtifactEntry:
+@dataclass
+class ArtifactManifestEntry:
     path: util.LogicalFilePathStr
-    ref: Optional[Union[util.FilePathStr, util.URIStr]]
-    digest: Union[util.B64MD5, util.URIStr, util.FilePathStr, util.ETag]
-    birth_artifact_id: Optional[str]
-    size: Optional[int]
-    extra: Dict
-    local_path: Optional[str]
+    digest: Union[B64MD5, util.URIStr, util.FilePathStr, ETag]
+    ref: Optional[Union[util.FilePathStr, util.URIStr]] = None
+    birth_artifact_id: Optional[str] = None
+    size: Optional[int] = None
+    extra: Dict = field(default_factory=dict)
+    local_path: Optional[str] = None
+
+    def __post_init__(self):
+        self.path = util.to_forward_slash_path(self.path)
+        if self.extra is None:
+            self.extra = {}
+        if self.local_path and self.size is None:
+            raise ValueError("size required when local_path specified")
 
     def parent_artifact(self) -> "Artifact":
         """
@@ -182,7 +144,9 @@ class ArtifactEntry:
         Raises:
             ValueError: If this artifact entry was not a reference.
         """
-        raise NotImplementedError
+        if self.ref is None:
+            raise ValueError("Only reference entries support ref_target().")
+        return self.ref
 
     def ref_url(self) -> str:
         """
@@ -221,6 +185,15 @@ class Artifact:
         """
         raise NotImplementedError
 
+    @property
+    def source_version(self) -> Optional[str]:
+        """
+        Returns:
+            (str) The artifact's version index under its parent artifact collection. This will return
+            a string with the format "v{number}".
+        """
+        raise NotImplementedError
+
     @property
     def name(self) -> str:
         """
@@ -446,7 +419,7 @@ class Artifact:
 
             Adding a directory without an explicit name:
             ```
-            artifact.add_dir('my_dir/', path='destination') # All files in `my_dir/` are added under `destination/`.
+            artifact.add_dir('my_dir/', name='destination') # All files in `my_dir/` are added under `destination/`.
             ```
 
         Raises:
@@ -459,7 +432,7 @@ class Artifact:
 
     def add_reference(
         self,
-        uri: Union[ArtifactEntry, str],
+        uri: Union[ArtifactManifestEntry, str],
         name: Optional[str] = None,
         checksum: bool = True,
         max_objects: Optional[int] = None,
@@ -556,7 +529,7 @@ class Artifact:
         """
         raise NotImplementedError
 
-    def get_path(self, name: str) -> ArtifactEntry:
+    def get_path(self, name: str) -> ArtifactManifestEntry:
         """
         Gets the path to the file located at the artifact relative `name`.
 
@@ -668,7 +641,7 @@ class Artifact:
 
         Arguments:
             root: (str, optional) The directory to verify. If None
-                artifact will be downloaded to './artifacts/<self.name>/'
+                artifact will be downloaded to './artifacts/self.name/'
 
         Raises:
             (ValueError): If the verification fails.
@@ -805,7 +778,7 @@ class StoragePolicy:
         pass
 
     def load_file(
-        self, artifact: Artifact, name: str, manifest_entry: ArtifactEntry
+        self, artifact: Artifact, name: str, manifest_entry: ArtifactManifestEntry
     ) -> str:
         raise NotImplementedError
 
@@ -813,7 +786,7 @@ class StoragePolicy:
         self,
         artifact_id: str,
         artifact_manifest_id: str,
-        entry: ArtifactEntry,
+        entry: ArtifactManifestEntry,
         preparer: "StepPrepare",
         progress_callback: Optional["progress.ProgressFn"] = None,
     ) -> bool:
@@ -828,7 +801,7 @@ class StoragePolicy:
         self,
         artifact: Artifact,
         name: str,
-        manifest_entry: ArtifactEntry,
+        manifest_entry: ArtifactManifestEntry,
         local: bool = False,
     ) -> str:
         raise NotImplementedError
@@ -846,7 +819,7 @@ class StorageHandler:
     def load_path(
         self,
         artifact: Artifact,
-        manifest_entry: ArtifactEntry,
+        manifest_entry: ArtifactManifestEntry,
         local: bool = False,
     ) -> Union[util.URIStr, util.FilePathStr]:
         """
@@ -862,7 +835,7 @@ class StorageHandler:
 
     def store_path(
         self, artifact, path, name=None, checksum=True, max_objects=None
-    ) -> Sequence[ArtifactEntry]:
+    ) -> Sequence[ArtifactManifestEntry]:
         """
         Stores the file or directory at the given path within the specified artifact.
 
@@ -882,7 +855,7 @@ class ArtifactsCache:
 
     def __init__(self, cache_dir):
         self._cache_dir = cache_dir
-        util.mkdir_exists_ok(self._cache_dir)
+        filesystem.mkdir_exists_ok(self._cache_dir)
         self._md5_obj_dir = os.path.join(self._cache_dir, "obj", "md5")
         self._etag_obj_dir = os.path.join(self._cache_dir, "obj", "etag")
         self._artifacts_by_id = {}
@@ -891,14 +864,14 @@ class ArtifactsCache:
         self._artifacts_by_client_id = {}
 
     def check_md5_obj_path(
-        self, b64_md5: util.B64MD5, size: int
+        self, b64_md5: B64MD5, size: int
     ) -> Tuple[util.FilePathStr, bool, "Opener"]:
-        hex_md5 = util.bytes_to_hex(base64.b64decode(b64_md5))
+        hex_md5 = b64_to_hex_id(b64_md5)
         path = os.path.join(self._cache_dir, "obj", "md5", hex_md5[:2], hex_md5[2:])
         opener = self._cache_opener(path)
         if os.path.isfile(path) and os.path.getsize(path) == size:
             return path, True, opener
-        util.mkdir_exists_ok(os.path.dirname(path))
+        filesystem.mkdir_exists_ok(os.path.dirname(path))
         return path, False, opener
 
     # TODO(spencerpearson): this method at least needs its signature changed.
@@ -906,7 +879,7 @@ class ArtifactsCache:
     def check_etag_obj_path(
         self,
         url: util.URIStr,
-        etag: util.ETag,
+        etag: ETag,
         size: int,
     ) -> Tuple[util.FilePathStr, bool, "Opener"]:
         hexhash = hashlib.sha256(
@@ -917,7 +890,7 @@ class ArtifactsCache:
         opener = self._cache_opener(path)
         if os.path.isfile(path) and os.path.getsize(path) == size:
             return path, True, opener
-        util.mkdir_exists_ok(os.path.dirname(path))
+        filesystem.mkdir_exists_ok(os.path.dirname(path))
         return path, False, opener
 
     def get_artifact(self, artifact_id):
@@ -1015,6 +988,24 @@ _artifacts_cache = None
 def get_artifacts_cache() -> ArtifactsCache:
     global _artifacts_cache
     if _artifacts_cache is None:
-        cache_dir = os.path.join(env.get_cache_dir(), "artifacts")
+        singleton = wandb.sdk.wandb_setup._WandbSetup._instance
+        cache_dir_base = (
+            wandb.run.settings.cache_dir
+            if wandb.run and wandb.run.settings.cache_dir
+            else singleton._settings.cache_dir
+            if singleton and singleton._settings.cache_dir
+            else env.get_cache_dir()
+        )
+        cache_dir = os.path.join(cache_dir_base, "artifacts")
         _artifacts_cache = ArtifactsCache(cache_dir)
     return _artifacts_cache
+
+
+def get_staging_dir() -> util.FilePathStr:
+    path = os.path.join(env.get_data_dir(), "artifacts", "staging")
+    filesystem.mkdir_exists_ok(path)
+    return os.path.abspath(os.path.expanduser(path))
+
+
+def get_new_staging_file() -> IO:
+    return tempfile.NamedTemporaryFile(dir=get_staging_dir(), delete=False)
