@@ -66,7 +66,7 @@ class InterfaceGrpc(InterfaceBase):
         return resp  # type: ignore
 
     def _communicate_run(
-        self, run: pb.RunRecord, timeout: int = None
+        self, run: pb.RunRecord, timeout: Optional[int] = None
     ) -> Optional[pb.RunUpdateResult]:
         assert self._stub
         self._assign(run)
@@ -78,6 +78,11 @@ class InterfaceGrpc(InterfaceBase):
         self._assign(run)
         _ = self._stub.RunUpdate(run)
 
+    def _publish_cancel(self, cancel: pb.CancelRequest) -> None:
+        assert self._stub
+        self._assign(cancel)
+        _ = self._stub.Cancel(cancel)
+
     def _publish_config(self, cfg: pb.ConfigRecord) -> None:
         assert self._stub
         self._assign(cfg)
@@ -170,6 +175,11 @@ class InterfaceGrpc(InterfaceBase):
         self._assign(link_artifact)
         _ = self._stub.LinkArtifact(link_artifact)
 
+    def _publish_use_artifact(self, use_artifact: pb.UseArtifactRecord) -> None:
+        assert self._stub
+        self._assign(use_artifact)
+        _ = self._stub.UseArtifact(use_artifact)
+
     def _communicate_artifact(
         self, log_artifact: pb.LogArtifactRequest
     ) -> MessageFuture:
@@ -234,6 +244,24 @@ class InterfaceGrpc(InterfaceBase):
         # TODO: implement
         return None
 
+    def _deliver_network_status(self, status: pb.NetworkStatusRequest) -> MailboxHandle:
+        assert self._stub
+        self._assign(status)
+        network_status_response = pb.NetworkStatusResponse()
+        response = pb.Response(network_status_response=network_status_response)
+        result = pb.Result(response=response)
+        handle = self._deliver(result)
+        return handle
+
+    def _deliver_stop_status(self, status: pb.StopStatusRequest) -> MailboxHandle:
+        assert self._stub
+        self._assign(status)
+        stop_status_response = pb.StopStatusResponse()
+        response = pb.Response(stop_status_response=stop_status_response)
+        result = pb.Result(response=response)
+        handle = self._deliver(result)
+        return handle
+
     def _communicate_stop_status(
         self, status: pb.StopStatusRequest
     ) -> Optional[pb.StopStatusResponse]:
@@ -375,5 +403,29 @@ class InterfaceGrpc(InterfaceBase):
         handle = self._deliver(result)
         return handle
 
+    def _deliver_request_run_status(
+        self, run_status: pb.RunStatusRequest
+    ) -> MailboxHandle:
+        assert self._stub
+        self._assign(run_status)
+        run_status_response = self._stub.RunStatus(run_status)
+        response = pb.Response(run_status_response=run_status_response)
+        result = pb.Result(response=response)
+        handle = self._deliver(result)
+        return handle
+
+    def _deliver_run_start(self, run_start: pb.RunStartRequest) -> MailboxHandle:
+        assert self._stub
+        self._assign(run_start)
+        try:
+            run_start_response = self._stub.RunStart(run_start)
+        except grpc.RpcError as e:
+            logger.info(f"RUNSTART TIMEOUT: {e}")
+            run_start_response = pb.RunStartResponse()
+        response = pb.Response(run_start_response=run_start_response)
+        result = pb.Result(response=response)
+        handle = self._deliver(result)
+        return handle
+
     def join(self) -> None:
         super().join()
