@@ -9,14 +9,17 @@ SettingsDict = Dict[str, Union[str, float, Tuple, None]]
 class SettingsStatic:
     # TODO(jhr): figure out how to share type defs with sdk/wandb_settings.py
     _offline: Optional[bool]
+    _sync: bool
     _disable_stats: Optional[bool]
     _disable_meta: Optional[bool]
+    _flow_control: bool
     _start_time: float
     _start_datetime: str
     _stats_pid: int
     _stats_sample_rate_seconds: float
     _stats_samples_to_average: int
     _stats_join_assets: bool
+    _stats_neuron_monitor_config_path: Optional[str]
     files_dir: str
     program_relpath: Optional[str]
     log_internal: str
@@ -28,6 +31,7 @@ class SettingsStatic:
     _jupyter_path: Optional[str]
     _jupyter_name: Optional[str]
     _jupyter_root: Optional[str]
+    _network_buffer: Optional[int]
     _require_service: Optional[str]
     _live_policy_rate_limit: Optional[int]
     resume: Optional[str]
@@ -50,6 +54,12 @@ class SettingsStatic:
     anonymous: Optional[str]
     host: Optional[str]
     username: Optional[str]
+    _executable: str
+    run_url: Optional[str]
+    run_name: Optional[str]
+    sync_file: str
+    _flow_control_disabled: bool
+    _flow_control_custom: bool
 
     # TODO(jhr): clean this up, it is only in SettingsStatic and not in Settings
     _log_level: int
@@ -78,5 +88,5 @@ class SettingsStatic:
     def __contains__(self, key: str) -> bool:
         return key in self.__dict__
 
-    def get(self, key: str, default: Any = None) -> Any:
+    def get(self, key: str, default: Optional[Any] = None) -> Any:
         return self.__dict__.get(key, default)
