@@ -8,7 +8,7 @@ from .abstract import AbstractBuilder
 __logger__ = logging.getLogger(__name__)
 
 
-_WANDB_BUILDERS: List[str] = ["kaniko", "docker"]
+_WANDB_BUILDERS: List[str] = ["kaniko", "docker", "noop"]
 
 
 def load_builder(builder_config: Dict[str, Any]) -> AbstractBuilder:
@@ -21,6 +21,10 @@ def load_builder(builder_config: Dict[str, Any]) -> AbstractBuilder:
         from .docker import DockerBuilder
 
         return DockerBuilder(builder_config)
+    elif builder_name == "noop":
+        from .noop import NoOpBuilder
+
+        return NoOpBuilder(builder_config)
     raise LaunchError(
         "Builder name not among available builders. Available builders: {} ".format(
             ",".join(_WANDB_BUILDERS)
