@@ -4,20 +4,22 @@ from typing import Any, Dict, List, Optional, Tuple
 
 import yaml
 
+import wandb
 from wandb.apis.internal import Api
 from wandb.errors import ExecutionError, LaunchError
 
 from ._project_spec import create_project_from_spec, fetch_and_validate_project
 from .agent import LaunchAgent
 from .builder import loader as builder_loader
+from .builder.build import construct_builder_args
 from .runner import loader
 from .runner.abstract import AbstractRun
 from .utils import (
     LAUNCH_CONFIG_FILE,
+    LAUNCH_DEFAULT_PROJECT,
     PROJECT_DOCKER_ARGS,
     PROJECT_SYNCHRONOUS,
     construct_launch_spec,
-    resolve_build_and_registry_config,
     validate_launch_spec_source,
 )
 
@@ -33,6 +35,7 @@ def resolve_agent_config(
 ) -> Tuple[Dict[str, Any], Api]:
     defaults = {
         "entity": api.default_entity,
+        "project": LAUNCH_DEFAULT_PROJECT,
         "max_jobs": 1,
         "queues": ["default"],
         "api_key": api.api_key,
@@ -41,7 +44,7 @@ def resolve_agent_config(
         "build": {},
         "runner": {},
     }
-
+    user_set_project = False
     resolved_config: Dict[str, Any] = defaults
     if os.path.exists(os.path.expanduser(LAUNCH_CONFIG_FILE)):
         config = {}
@@ -51,9 +54,12 @@ def resolve_agent_config(
                 print(config)
             except yaml.YAMLError as e:
                 raise LaunchError(f"Invalid launch agent config: {e}")
+        if config.get("project") is not None:
+            user_set_project = True
         resolved_config.update(dict(config))
     if os.environ.get("WANDB_PROJECT") is not None:
         resolved_config.update({"project": os.environ.get("WANDB_PROJECT")})
+        user_set_project = True
     if os.environ.get("WANDB_ENTITY") is not None:
         resolved_config.update({"entity": os.environ.get("WANDB_ENTITY")})
     if os.environ.get("WANDB_API_KEY") is not None:
@@ -67,6 +73,7 @@ def resolve_agent_config(
 
     if project is not None:
         resolved_config.update({"project": project})
+        user_set_project = True
     if entity is not None:
         resolved_config.update({"entity": entity})
     if max_jobs is not None:
@@ -87,6 +94,11 @@ def resolve_agent_config(
         )
         api = Api(default_settings=settings)
 
+    if user_set_project:
+        wandb.termwarn(
+            "Specifying a project for the launch agent is deprecated. Please use queues found in the Launch application at https://wandb.ai/launch."
+        )
+
     return resolved_config, api
 
 
@@ -115,6 +127,7 @@ def _run(
     cuda: Optional[bool],
     api: Api,
     run_id: Optional[str],
+    repository: Optional[str],
 ) -> AbstractRun:
     """Helper that delegates to the project-running method corresponding to the passed-in backend."""
     launch_spec = construct_launch_spec(
@@ -133,6 +146,7 @@ def _run(
         launch_config,
         cuda,
         run_id,
+        repository,
     )
     validate_launch_spec_source(launch_spec)
     launch_project = create_project_from_spec(launch_spec, api)
@@ -141,25 +155,19 @@ def _run(
     # construct runner config.
     runner_config: Dict[str, Any] = {}
     runner_config[PROJECT_SYNCHRONOUS] = synchronous
-    build_config: Optional[Dict[str, Any]] = None
-    registry_config = None
-    if launch_config is not None:
-        given_docker_args = launch_config.get("docker", {}).get("args", {})
-        build_config = launch_config.get("build")
-        registry_config = launch_config.get("registry")
-        runner_config[PROJECT_DOCKER_ARGS] = given_docker_args
-    else:
-        runner_config[PROJECT_DOCKER_ARGS] = {}
 
-    default_launch_config = None
+    if repository:  # override existing registry with CLI arg
+        launch_config = launch_config or {}
+        registry = launch_config.get("registry", {})
+        registry["url"] = repository
+        launch_config["registry"] = registry
 
-    if os.path.exists(os.path.expanduser(LAUNCH_CONFIG_FILE)):
-        with open(os.path.expanduser(LAUNCH_CONFIG_FILE)) as f:
-            default_launch_config = yaml.safe_load(f)
-
-    build_config, registry_config = resolve_build_and_registry_config(
-        default_launch_config, build_config, registry_config
+    given_docker_args, build_config, registry_config = construct_builder_args(
+        launch_config,
     )
+
+    runner_config[PROJECT_DOCKER_ARGS] = given_docker_args
+
     builder = builder_loader.load_builder(build_config)
     backend = loader.load_backend(resource, api, runner_config)
     if backend:
@@ -170,9 +178,7 @@ def _run(
         return submitted_run
     else:
         raise ExecutionError(
-            "Unavailable backend {}, available backends: {}".format(
-                resource, ", ".join(loader.WANDB_RUNNERS)
-            )
+            f"Unavailable backend {resource}, available backends: {', '.join(loader.WANDB_RUNNERS)}"
         )
 
 
@@ -184,7 +190,7 @@ def run(
     version: Optional[str] = None,
     parameters: Optional[Dict[str, Any]] = None,
     name: Optional[str] = None,
-    resource: str = "local",
+    resource: Optional[str] = None,
     resource_args: Optional[Dict[str, Any]] = None,
     project: Optional[str] = None,
     entity: Optional[str] = None,
@@ -193,6 +199,7 @@ def run(
     synchronous: Optional[bool] = True,
     cuda: Optional[bool] = None,
     run_id: Optional[str] = None,
+    repository: Optional[str] = None,
 ) -> AbstractRun:
     """Run a W&B launch experiment. The project can be wandb uri or a Git URI.
 
@@ -206,7 +213,7 @@ def run(
     parameters: Parameters (dictionary) for the entry point command. Defaults to using the
         the parameters used to run the original run.
     name: Name run under which to launch the run.
-    resource: Execution backend for the run: W&B provides built-in support for "local" backend
+    resource: Execution backend for the run.
     resource_args: Resource related arguments for launching runs onto a remote backend.
         Will be stored on the constructed launch config under ``resource_args``.
     project: Target project to send launched run to
@@ -214,7 +221,7 @@ def run(
     config: A dictionary containing the configuration for the run. May also contain
     resource specific arguments under the key "resource_args".
     synchronous: Whether to block while waiting for a run to complete. Defaults to True.
-        Note that if ``synchronous`` is False and ``backend`` is "local", this
+        Note that if ``synchronous`` is False and ``backend`` is "local-container", this
         method will return, but the current process will block when exiting until
         the local run completes. If the current process is interrupted, any
         asynchronous runs launched via this method will be terminated. If
@@ -222,7 +229,7 @@ def run(
         error out as well.
     cuda: Whether to build a CUDA-enabled docker image or not
     run_id: ID for the run (To ultimately replace the :name: field)
-
+    repository: string name of repository path for remote registry
 
     Example:
         import wandb
@@ -245,6 +252,10 @@ def run(
     if config is None:
         config = {}
 
+    # default to local container for runs without a queue
+    if resource is None:
+        resource = "local-container"
+
     submitted_run_obj = _run(
         uri=uri,
         job=job,
@@ -262,6 +273,7 @@ def run(
         cuda=cuda,
         api=api,
         run_id=run_id,
+        repository=repository,
     )
 
     return submitted_run_obj
