@@ -12,6 +12,7 @@ from wandb import Settings
 from wandb.apis.internal import Api
 from wandb.errors import CommError
 from wandb.sdk.launch.builder.abstract import AbstractBuilder
+from wandb.sdk.lib import runid
 
 from .._project_spec import LaunchProject
 
@@ -118,7 +119,7 @@ class AbstractRunner(ABC):
         self._api = api
         self.backend_config = backend_config
         self._cwd = os.getcwd()
-        self._namespace = wandb.util.generate_id()
+        self._namespace = runid.generate_id()
 
     def find_executable(
         self, cmd: str
