@@ -98,6 +98,7 @@ class LocalContainerRunner(AbstractRunner):
             env_vars["WANDB_BASE_URL"] = f"http://host.docker.internal:{port}"
         elif _is_wandb_dev_uri(self._api.settings("base_url")):
             env_vars["WANDB_BASE_URL"] = "http://host.docker.internal:9002"
+
         if launch_project.docker_image:
             # user has provided their own docker image
             image_uri = launch_project.image_name
@@ -168,7 +169,7 @@ def get_docker_command(
     image: str,
     env_vars: Dict[str, str],
     entry_cmd: List[str],
-    docker_args: Dict[str, Any] = None,
+    docker_args: Optional[Dict[str, Any]] = None,
 ) -> List[str]:
     """Constructs the docker command using the image and docker args.
 
