@@ -3,6 +3,7 @@ config.
 """
 
 import logging
+from typing import Optional
 
 import wandb
 from wandb.util import (
@@ -39,7 +40,7 @@ class Config:
 
     Examples:
         Basic usage
-        ```python
+        ```
         wandb.config.epochs = 4
         wandb.init()
         for x in range(wandb.config.epochs):
@@ -47,14 +48,14 @@ class Config:
         ```
 
         Using wandb.init to set config
-        ```python
+        ```
         wandb.init(config={"epochs": 4, "batch_size": 32})
         for x in range(wandb.config.epochs):
             # train
         ```
 
         Nested configs
-        ```python
+        ```
         wandb.config['train']['epochs'] = 4
         wandb.init()
         for x in range(wandb.config['train']['epochs']):
@@ -62,7 +63,7 @@ class Config:
         ```
 
         Using absl flags
-        ```python
+        ```
         flags.DEFINE_string(‘model’, None, ‘model to run’) # name, default, help
         wandb.config.update(flags.FLAGS) # adds all absl flags to config
         ```
@@ -73,8 +74,14 @@ class Config:
         wandb.config.epochs = 4
 
         parser = argparse.ArgumentParser()
-        parser.add_argument('-b', '--batch-size', type=int, default=8, metavar='N',
-                            help='input batch size for training (default: 8)')
+        parser.add_argument(
+            "-b",
+            "--batch-size",
+            type=int,
+            default=8,
+            metavar="N",
+            help="input batch size for training (default: 8)",
+        )
         args = parser.parse_args()
         wandb.config.update(args)
         ```
@@ -82,8 +89,8 @@ class Config:
         Using TensorFlow flags (deprecated in tensorflow v2)
         ```python
         flags = tf.app.flags
-        flags.DEFINE_string('data_dir', '/tmp/data')
-        flags.DEFINE_integer('batch_size', 128, 'Batch size.')
+        flags.DEFINE_string("data_dir", "/tmp/data")
+        flags.DEFINE_integer("batch_size", 128, "Batch size.")
         wandb.config.update(flags.FLAGS)  # adds all of the tensorflow flags to config
         ```
     """
@@ -224,7 +231,7 @@ class Config:
         self,
         config_dict,
         allow_val_change=None,
-        ignore_keys: set = None,
+        ignore_keys: Optional[set] = None,
     ):
         sanitized = {}
         self._raise_value_error_on_nested_artifact(config_dict)
