@@ -1,13 +1,15 @@
 import os
 from functools import wraps
-from typing import Any, Callable, TypeVar, cast
+from typing import Any, Callable, Dict, TypeVar, cast
 
 FuncT = TypeVar("FuncT", bound=Callable[..., Any])
 
-requirement_env_var_mapping = {"report-editing:v0": "WANDB_REQUIRE_REPORT_EDITING_V0"}
+requirement_env_var_mapping: Dict[str, str] = {
+    "report-editing:v0": "WANDB_REQUIRE_REPORT_EDITING_V0"
+}
 
 
-def requires(requirement: str) -> FuncT:
+def requires(requirement: str) -> FuncT:  # type: ignore
     """
     The decorator for gating features.
     """
@@ -48,7 +50,3 @@ class RequiresMixin:
             raise Exception(
                 f'You must explicitly enable this feature with `wandb.require("{self.requirement})"'
             )
-
-
-class RequiresReportEditingMixin(RequiresMixin):
-    requirement = "report-editing:v0"
