@@ -30,8 +30,8 @@ def _get_sweep_url(api, sweep_id):
 
 def sweep(
     sweep: Union[dict, Callable],
-    entity: str = None,
-    project: str = None,
+    entity: Optional[str] = None,
+    project: Optional[str] = None,
 ) -> str:
     """Initialize a hyperparameter sweep.
 
