@@ -6,7 +6,6 @@ from sklearn import metrics
 from sklearn.utils.multiclass import unique_labels
 
 import wandb
-
 from wandb.sklearn import utils
 
 # ignore all future warnings
@@ -81,7 +80,11 @@ def make_table(cm, pred_classes, true_classes, labels):
             true = true_classes[j]
         data.append([pred, true, cm[i, j]])
         count += 1
-        if utils.check_against_limit(count, "confusion_matrix", utils.chart_limit,):
+        if utils.check_against_limit(
+            count,
+            "confusion_matrix",
+            utils.chart_limit,
+        ):
             break
 
     table = wandb.Table(columns=["Predicted", "Actual", "Count"], data=data)
