@@ -15,8 +15,9 @@ from urllib.parse import quote as url_quote
 import wandb
 from wandb.proto import wandb_internal_pb2  # type: ignore
 from wandb.sdk.interface.interface_queue import InterfaceQueue
-from wandb.sdk.internal import datastore, handler, sender, tb_watcher
-from wandb.util import check_and_warn_old, mkdir_exists_ok
+from wandb.sdk.internal import context, datastore, handler, sender, tb_watcher
+from wandb.sdk.lib import filesystem
+from wandb.util import check_and_warn_old
 
 WANDB_SUFFIX = ".wandb"
 SYNCED_SUFFIX = ".synced"
@@ -156,8 +157,13 @@ class SyncThread(threading.Thread):
         record_q = queue.Queue()
         sender_record_q = queue.Queue()
         new_interface = InterfaceQueue(record_q)
+        context_keeper = context.ContextKeeper()
         send_manager = sender.SendManager(
-            send_manager._settings, sender_record_q, queue.Queue(), new_interface
+            settings=send_manager._settings,
+            record_q=sender_record_q,
+            result_q=queue.Queue(),
+            interface=new_interface,
+            context_keeper=context_keeper,
         )
         record = send_manager._interface._make_record(run=proto_run)
         settings = wandb.Settings(
@@ -168,10 +174,16 @@ class SyncThread(threading.Thread):
         )
 
         handle_manager = handler.HandleManager(
-            settings, record_q, None, False, sender_record_q, None, new_interface
+            settings=settings,
+            record_q=record_q,
+            result_q=None,
+            stopped=False,
+            writer_q=sender_record_q,
+            interface=new_interface,
+            context_keeper=context_keeper,
         )
 
-        mkdir_exists_ok(settings.files_dir)
+        filesystem.mkdir_exists_ok(settings.files_dir)
         send_manager.send_run(record, file_dir=settings.files_dir)
         watcher = tb_watcher.TBWatcher(settings, proto_run, new_interface, True)
 
