@@ -6,24 +6,23 @@ Usage:
     trigger.call('on_something', *args, **kwargs)
     trigger.unregister('on_something', func)
 """
-
+from typing import Any, Callable
 
 _triggers = {}
 
 
 def reset():
-    global triggers
-    _triggers = {}
+    _triggers.clear()
 
 
-def register(event_str, func):
-    _triggers.setdefault(event_str, []).append(func)
+def register(event: str, func: Callable):
+    _triggers.setdefault(event, []).append(func)
 
 
-def call(event_str, *args, **kwargs):
+def call(event_str: str, *args: Any, **kwargs: Any):
     for func in _triggers.get(event_str, []):
         func(*args, **kwargs)
 
 
-def unregister(event_str, func):
-    _triggers[event_str].remove(func)
+def unregister(event: str, func: Callable):
+    _triggers[event].remove(func)
